package PowerListingFieldEditor::CMS::ContentDataAsset;

use strict;
use warnings;

use MT::Util
    qw( epoch2ts encode_url format_ts relative_date perl_sha1_digest_hex);

my $default_thumbnail_size = 60;

# "選択"クリック時に起動
sub dialog_list_asset {
    my $app = shift;

    return dialog_asset_modal( $app, @_ ) unless $app->param('json');

# これより下は実行されない

    my $blog_id = $app->param('blog_id');
    my $mode_userpic = $app->param('upload_mode') || '';
    return $app->return_to_dashboard( redirect => 1 )
        if !$blog_id && $mode_userpic ne 'upload_userpic';

    my $blog_class = $app->model('blog');
    my $blog;
    $blog = $blog_class->load($blog_id) if $blog_id;

    my $edit_field = $app->param('edit_field') || '';
    if ( $edit_field =~ m/^customfield_.*$/ ) {
        return $app->permission_denied()
            unless $app->permissions;
    }
    else {
        return $app->permission_denied()
            if $blog_id && !$app->can_do('access_to_insert_asset_list');
    }

    my $asset_class = $app->model('asset') or return;
    my %terms;
    my %args = ( sort => 'created_on', direction => 'descend' );

    my $class_filter;
    my $filter = $app->param('filter') || '';
    if ( $filter eq 'class' ) {
        $class_filter = $app->param('filter_val');
    }
    elsif ( $filter eq 'userpic' ) {
        $class_filter      = 'image';
        $terms{created_by} = $app->param('filter_val');
        $terms{blog_id}    = 0;

        my $tag = MT->model('tag')
            ->load( { name => '@userpic' }, { binary => { name => 1 } } );
        if ($tag) {
            require MT::ObjectTag;
            $args{'join'} = MT::ObjectTag->join_on(
                'object_id',
                {   tag_id            => $tag->id,
                    object_datasource => MT::Asset->datasource
                },
                { unique => 1 }
            );
        }
    }

    $app->add_breadcrumb( $app->translate("Files") );

    if ($blog_id) {
        my $blog_ids = $app->_load_child_blog_ids($blog_id);
        push @$blog_ids, $blog_id;
        $terms{blog_id} = $blog_ids;
    }

    my $hasher = build_asset_hasher(
        $app,
        PreviewWidth  => 120,
        PreviewHeight => 120
    );

    if ($class_filter) {
        my $asset_pkg = MT::Asset->class_handler($class_filter);
        $terms{class} = $asset_pkg->type_list;
    }
    else {
        $terms{class} = '*';    # all classes
    }

    # identifier => name
    my $classes = MT::Asset->class_labels;
    my @class_loop;
    foreach my $class ( keys %$classes ) {
        next if $class eq 'asset';
        push @class_loop,
            {
            class_id    => $class,
            class_label => $classes->{$class},
            };
    }

    # Now, sort it
    @class_loop
        = sort { $a->{class_label} cmp $b->{class_label} } @class_loop;

    my $dialog    = $app->param('dialog')    ? 1 : 0;
    my $no_insert = $app->param('no_insert') ? 1 : 0;
    my %carry_params
        = map { $_ => $app->param($_) || '' }
        (
        qw( edit_field upload_mode require_type next_mode asset_select can_multi )
        );
    $carry_params{'user_id'} = $app->param('filter_val')
        if $filter eq 'userpic';
    _set_start_upload_params( $app, \%carry_params )
        if $app->can_do('upload');
    my $ext_from = $app->param('ext_from');
    my $ext_to   = $app->param('ext_to');

    # Check directory for thumbnail image
    _check_thumbnail_dir( $app, \%carry_params );

    $app->listing(
        {   terms    => \%terms,
            args     => \%args,
            type     => 'asset',
            code     => $hasher,
            template => 'include/async_asset_list_for_cd.tmpl',
            params   => {
                (   $blog
                    ? ( blog_id      => $blog_id,
                        blog_name    => $blog->name || '',
                        edit_blog_id => $blog_id,
                        ( $blog->is_blog ? ( blog_view => 1 ) : () ),
                        )
                    : (),
                ),
                is_image => defined $class_filter
                    && $class_filter eq 'image' ? 1 : 0,
                dialog_view      => 1,
                dialog           => $dialog,
                no_insert        => $no_insert,
                search_label     => MT::Asset->class_label_plural,
                search_type      => 'asset',
                class_loop       => \@class_loop,
                can_delete_files => $app->can_do('delete_asset_file') ? 1 : 0,
                nav_assets       => 1,
                panel_searchable => 1,
                saved_deleted    => $app->param('saved_deleted') ? 1 : 0,
                object_type      => 'asset',
                (     ( $ext_from && $ext_to )
                    ? ( ext_from => $ext_from, ext_to => $ext_to )
                    : ()
                ),
                dir_separator => MT::Util::dir_separator,
                %carry_params,
            },
        }
    );
}

sub dialog_asset_modal {
    my $app = shift;

    my $blog_id = $app->param('blog_id');
    my $mode_userpic = $app->param('upload_mode') || '';
    return $app->return_to_dashboard( redirect => 1 )
        if !$blog_id && $mode_userpic ne 'upload_userpic';

    my $blog_class = $app->model('blog');
    my $blog;
    $blog = $blog_class->load($blog_id) if $blog_id;

    my %param;
    _set_start_upload_params( $app, \%param );

    if (   $app->param('edit_field')
        && $app->param('edit_field') =~ m/^customfield_.*$/ )
    {
        return $app->permission_denied()
            unless $app->permissions;
    }
    else {
        return $app->permission_denied()
            if $blog_id && !$app->can_do('access_to_insert_asset_list');
    }

$param{obj_id} = $app->param('obj_id');
$param{basename} = $app->param('basename');

    $param{can_multi} = 1
        if ( $app->param('upload_mode') || '' ) ne 'upload_userpic'
        && $app->param('can_multi');

    $param{filter} = $app->param('filter')
        if defined $app->param('filter');
    $param{filter_val} = $app->param('filter_val')
        if defined $app->param('filter_val');
    $param{search} = $app->param('search') if defined $app->param('search');
    $param{edit_field} = $app->param('edit_field')
        if defined $app->param('edit_field');
    $param{next_mode}    = $app->param('next_mode');
    $param{no_insert}    = $app->param('no_insert') ? 1 : 0;
    $param{asset_select} = $app->param('asset_select');
    $param{require_type} = $app->param('require_type');

    if ($blog_id) {
        $param{blog_id}      = $blog_id;
        $param{edit_blog_id} = $blog_id,;
    }

    $param{upload_mode} = $mode_userpic;
    if ($mode_userpic) {
        $param{user_id}      = $param{filter_val} || $app->user->id;
        $param{require_type} = 'image';
        $param{'is_image'}   = 1;
        $param{can_upload}   = 1;
    }

    if ( $param{require_type} ) {
        my $req_class = $app->model( $param{require_type} );
        $param{require_type_label} = $req_class->class_label;
    }

    require MT::Asset;
    my $subclasses = MT::Asset->list_subclasses;

    my @class_filters;
    foreach my $k (@$subclasses) {
        my $c = $k->{class};
        push @class_filters,
            {
            key   => $k->{type},
            label => $c->class_label_plural,
            };
    }
    $param{class_filter_loop} = \@class_filters if @class_filters;

    # Set directory separator
    $param{dir_separator} = MT::Util::dir_separator;

    if ( my $content_field_id = $app->param('content_field_id') ) {
        require MT::ContentField;
        if ( my $content_field = MT::ContentField->load($content_field_id) ) {
            $param{content_field_id} = $content_field_id;
            my $options = $content_field->options;
            $param{can_multi}  = $options->{multiple}     ? 1 : 0;
            $param{can_upload} = $options->{allow_upload} ? 1 : 0;
        }
    }

    $app->load_tmpl( 'dialog/asset_modal_for_cd.tmpl', \%param );
}

sub _make_upload_destinations {
    my $app = shift;
    my ( $blog, $real_path ) = @_;

    my @dest_root;
    my $class_label = $app->translate('Site');

    require POSIX;
    my $user_basename;
    my $y;
    my $ym;
    my $ymd;
    my $now = MT::Util::offset_time(time);

    if ($real_path) {
        $user_basename = $app->user->basename;
        $y             = POSIX::strftime( "%Y", gmtime($now) );
        $ym            = POSIX::strftime( "%Y/%m", gmtime($now) );
        $ymd           = POSIX::strftime( "%Y/%m/%d", gmtime($now) );
    }
    else {
        $user_basename = $app->translate('basename of user');
        $y             = 'yyyy';
        $ym            = 'yyyy/mm';
        $ymd           = 'yyyy/mm/dd';
    }

    push @dest_root,
        {
        label => $app->translate( '<[_1] Root>', $class_label ),
        path  => '%s',
        };
    push @dest_root,
        {
        label => $app->translate(
            '<[_1] Root>/[_2]',
            $class_label, $user_basename
        ),
        path => '%s/%u',
        };
    push @dest_root,
        {
        label => $app->translate( '<[_1] Root>/[_2]', $class_label, $y ),
        path  => '%s/%y',
        };
    push @dest_root,
        {
        label => $app->translate( '<[_1] Root>/[_2]', $class_label, $ym ),
        path  => '%s/%y/%m',
        };
    push @dest_root,
        {
        label => $app->translate( '<[_1] Root>/[_2]', $class_label, $ymd ),
        path  => '%s/%y/%m/%d',
        };

    if ( $blog->column('archive_path') ) {
        $class_label = MT->translate('Archive');
        push @dest_root,
            {
            label => $app->translate( '<[_1] Root>', $class_label ),
            path  => '%a',
            };
        push @dest_root,
            {
            label => $app->translate(
                '<[_1] Root>/[_2]',
                $class_label, $user_basename
            ),
            path => '%a/%u',
            };
        push @dest_root,
            {
            label => $app->translate( '<[_1] Root>/[_2]', $class_label, $y ),
            path  => '%a/%y',
            };
        push @dest_root,
            {
            label => $app->translate( '<[_1] Root>/[_2]', $class_label, $ym ),
            path  => '%a/%y/%m',
            };
        push @dest_root,
            {
            label =>
                $app->translate( '<[_1] Root>/[_2]', $class_label, $ymd ),
            path => '%a/%y/%m/%d',
            };
    }

    if ( $blog->upload_destination ) {
        if ( my @selected
            = grep { $blog->upload_destination eq $_->{path} } @dest_root )
        {
            $_->{selected} = 1 for @selected;
        }
        else {
            my $label = $blog->upload_destination;
            if ($real_path) {

                # Replace %s and %a.
                if ( $label =~ /^%s/ ) {
                    my $site_root
                        = $app->translate( '<[_1] Root>',
                        $blog->class_label );
                    $label =~ s/^%s/$site_root/;
                }
                elsif ( $label =~ /^%a/ ) {
                    my $archive_root = $app->translate( '<[_1] Root>',
                        MT->translate('Archive') );
                    $label =~ s/^%a/$archive_root/;
                }

                # Replace %u, %y, %m and %d.
                my $u = $app->user->basename;
                my $y = POSIX::strftime( "%Y", gmtime($now) );
                my $m = POSIX::strftime( "%m", gmtime($now) );
                my $d = POSIX::strftime( "%d", gmtime($now) );
                $label =~ s/%u/$u/g;
                $label =~ s/%y/$y/g;
                $label =~ s/%m/$m/g;
                $label =~ s/%d/$d/g;
            }

            unshift @dest_root,
                {
                label    => $label,
                path     => $blog->upload_destination,
                selected => 1,
                };
        }
    }

    push @dest_root, { label => $app->translate('Custom...') };

    return @dest_root;
}

sub _set_start_upload_params {

    my $app = shift;
    my ($param) = @_;

    if ( my $perms = $app->permissions ) {
        my $blog_id = $app->param('blog_id');
        if ($blog_id) {
            my $blog = MT->model('blog')->load($blog_id)
                or return $app->error(
                $app->translate( 'Cannot load blog #[_1].', $blog_id ) );

            # Make a list of upload destination
            my @dest_root = _make_upload_destinations( $app, $blog, 1 );
            $param->{destination_loop} = \@dest_root;

            # Set default upload options
            $param->{allow_to_change_at_upload}
                = defined $blog->allow_to_change_at_upload
                ? $blog->allow_to_change_at_upload
                : 1;
            if ( !$param->{allow_to_change_at_upload} ) {
                foreach my $opt ( grep { $_->{selected} } @dest_root ) {
                    $param->{upload_destination_label} = $opt->{label};
                    $param->{upload_destination_value} = $opt->{path};
                }
            }
            $param->{destination}         = $blog->upload_destination;
            $param->{extra_path}          = $blog->extra_path;
            $param->{operation_if_exists} = $blog->operation_if_exists;
            $param->{normalize_orientation}
                = defined $blog->normalize_orientation
                ? $blog->normalize_orientation
                : 1;
            $param->{auto_rename_non_ascii}
                = defined $blog->auto_rename_non_ascii
                ? $blog->auto_rename_non_ascii
                : 1;
        }
    }
    else {
        $param->{normalize_orientation} = 1;
        $param->{auto_rename_non_ascii} = 1;
    }

    $param->{max_upload_size} = $app->config->CGIMaxUpload;

    $param;
}

# ダイアログで"挿入"クリック時に起動(オプションなしの場合は途中でinsert_asset起動）
sub dialog_insert_options {
    my $app    = shift;
    my (%args) = @_;
    my $assets = $args{assets};

    # Validate magic token
    $app->validate_magic() or return;

    # Load assets
    if ( !$assets ) {
        my $ids = $app->param('id');
        return $app->errtrans('Invalid request.') unless $ids;

        my @ids = split ',', $ids;
        return $app->errtrans('Invalid request.') unless @ids;
        my @assets = $app->model('asset')->load( { id => \@ids } );

        # Sort by @ids order.
        my %assets = map { $_->id => $_ } @assets;
        @assets = map { $assets{$_} } @ids;

        $assets = \@assets;
    }
    $assets = [$assets] if 'ARRAY' ne ref $assets;

    # Should not allow to insert asset from other site.
    my $blog_id = $app->param('blog_id');
    my $blog    = MT->model('blog')->load($blog_id)
        or return $app->errtrans( "Cannot load blog #[_1].", $blog_id );
    my %blog_ids;
    if ( !$blog->is_blog ) {
        %blog_ids = map { $_->id => 1 } @{ $blog->blogs };
    }
    $blog_ids{$blog_id} = 1;
    foreach my $a (@$assets) {
        return $app->errtrans('Invalid request.')
            unless defined $blog_ids{ $a->blog_id };
    }

    # If no_insert option provided, should not displaying insert option
    if ( $app->param('no_insert') || $app->param('direct_asset_insert') ) {
        return insert_asset( $app, { assets => $assets } );
    }

    # Permission check
    my $perms = $app->permissions
        or return $app->errtrans('No permissions');
    return $app->errtrans('No permissions')
        unless $perms->can_do('insert_asset');

    # Make a insert option loop
    my $options_loop;
    foreach my $a (@$assets) {
        my $param = {
            id        => $a->id,
            filename  => $a->file_name,
            url       => $a->url,
            label     => $a->label,
            thumbnail => _make_thumbnail_url(
                $a, { size => $default_thumbnail_size }
            ),
            thumbnail_type => $a->class eq 'video' ? 'movie' : $a->class,
            class_label => $a->class_label,
        };
        my $html = $a->insert_options($param) || '';
        $param->{options} = $html;
        push @$options_loop, $param;
    }

    my %param;
    $param{options_loop} = $options_loop;
    $param{edit_field}   = $app->param('edit_field');
    $param{new_entry}    = $app->param('asset_select') ? 0 : 1;

    $app->load_tmpl( 'dialog/multi_asset_options.tmpl', \%param );
}

sub insert_asset {
    my $app = shift;
    my ($param) = @_;

my $obj_id = $app->param('obj_id');
my $basename = $app->param('basename');

    $app->validate_magic() or return;

    my $edit_field = $app->param('edit_field') || '';
    if ( $edit_field =~ m/^customfield_.*$/ ) {
        return $app->permission_denied()
            unless $app->permissions;
    }
    else {
        return $app->permission_denied()
            unless $app->can_do('insert_asset');
    }

    require MT::Asset;
    my $text;
    my $assets;
    if ( $app->param('no_insert') ) {
        $text   = '';
        $assets = $param->{assets};
    }
    elsif ( $app->param('direct_asset_insert') ) {
        $assets = $param->{assets};
        foreach my $a (@$assets) {
            my %param;
            $param{wrap_text} = 1;
            $param{new_entry} = $app->param('new_entry') ? 1 : 0;

            $a->on_upload( \%param );
            $param{enclose} = $edit_field =~ /^customfield/ ? 1 : 0;
            my $html = $a->as_html( \%param );
            return $app->error( $a->error ) unless defined $html;

            $text .= $html;
        }
    }
    else {
        # Parse JSON.
        my $prefs = $app->param('prefs_json');
        $prefs =~ s/^"|"$//g;
        $prefs =~ s/\\"/"/g;
        $prefs =~ s/\\\\/\\/g;
        $prefs = eval { MT::Util::from_json($prefs) };
        if ( !$prefs ) {
            return $app->errtrans('Invalid request.');
        }

        foreach my $item (@$prefs) {
            my $id = $item->{id};
            return $app->errtrans('Invalid request.')
                unless $id;
            my $asset = MT->model('asset')->load($id)
                or return $app->errtrans( 'Cannot load asset #[_1]', $id );
            my %param;
            foreach my $k ( keys %$item ) {
                my $name = $k;
                if ( $k =~ m/(.*)[-|_]$id/ig ) {
                    $name = $1;
                }
                $param{$name} = $item->{$k};
            }
            $param{wrap_text} = 1;
            $param{new_entry} = $app->param('new_entry') ? 1 : 0;

            $asset->on_upload( \%param );
            $param{enclose} = $edit_field =~ /^customfield/ ? 1 : 0;
            my $html = $asset->as_html( \%param );
            return $app->error( $asset->error ) unless defined $html;

            $text .= $html;
            push @$assets, $asset;
        }
    }

    my $can_multi;
    my $content_field_id = $app->param('content_field_id');
    if ($content_field_id) {
        require MT::ContentField;
        if ( my $content_field = MT::ContentField->load($content_field_id) ) {
            my $options = $content_field->options;
            $can_multi = $options->{multiple} ? 1 : 0;
        }
        else {
            $content_field_id = undef;
        }
    }

    if ($content_field_id) {
        my @assets_data;
        my $hasher = build_asset_hasher(
            $app,
            PreviewWidth  => 80,
            PreviewHeight => 80,
        );
        for my $obj (@$assets) {
            my $row = $obj->get_values;
            $hasher->( $obj, $row );
            push @assets_data,
                {
                asset_dimensions     => $row->{'Actual Dimensions'},
                asset_file_name      => $row->{file_name},
                asset_id             => $row->{id},
                asset_label          => $row->{label},
                asset_preview_url    => $row->{preview_url},
                asset_preview_height => $row->{preview_height},
                asset_preview_width  => $row->{preview_width},
                asset_type           => $row->{class},
                };
        }
        return $app->load_tmpl(
            'dialog/asset_field_insert_for_cd.tmpl',
            {   assets           => \@assets_data,
                can_multi        => $can_multi,
                content_field_id => $content_field_id,
obj_id => $obj_id,
basename => $basename,
            }
        );
    }
    else {
        my $tmpl = $app->load_tmpl(
            'dialog/asset_insert_for_cd.tmpl',
            {   upload_html => $text || '',
                edit_field => $edit_field,
            },
        );
        my $ctx = $tmpl->context;
        $ctx->stash( 'assets', $assets );
        return $tmpl;
    }
}

sub build_asset_hasher {
    my $app = shift;
    my (%param) = @_;
    my ($default_thumb_width,   $default_thumb_height,
        $default_preview_width, $default_preview_height
    ) = @param{qw( ThumbWidth ThumbHeight PreviewWidth PreviewHeight )};

    require File::Basename;
    require JSON;
    my %blogs;
    return sub {
        my ( $obj, $row, %param ) = @_;
        my ( $thumb_width, $thumb_height )
            = @param{qw( ThumbWidth ThumbHeight )};
        $row->{id} = $obj->id;
        my $blog = $blogs{ $obj->blog_id } ||= $obj->blog;
        $row->{blog_name} = $blog ? $blog->name : '-';
        $row->{url} = $obj->url;    # this has to be called to calculate
        $row->{asset_type}        = $obj->class_type;
        $row->{asset_class_label} = $obj->class_label;
        my $file_path = $obj->file_path;    # has to be called to calculate
        my $meta      = $obj->metadata;

        require MT::FileMgr;
        my $fmgr = MT::FileMgr->new('Local');
        ## TBD: Make sure $file_path is file, not directory.
        if ( $file_path && $fmgr->exists($file_path) ) {
            $row->{file_path} = $file_path;
            $row->{file_name} = File::Basename::basename($file_path);
            my $size = $fmgr->file_size($file_path);
            $row->{file_size} = $size;
            if ( $size < 1024 ) {
                $row->{file_size_formatted} = sprintf( "%d Bytes", $size );
            }
            elsif ( $size < 1024000 ) {
                $row->{file_size_formatted}
                    = sprintf( "%.1f KB", $size / 1024 );
            }
            else {
                $row->{file_size_formatted}
                    = sprintf( "%.1f MB", $size / 1024000 );
            }
            $meta->{'file_size'} = $row->{file_size_formatted};
        }
        else {
            $row->{file_is_missing} = 1 if $file_path;
        }
        $meta->{file_name} = $row->{file_name};
        $row->{file_label} 
            = $row->{label} 
            = $obj->label
            || $row->{file_name}
            || $app->translate('Untitled');

        if ( $obj->has_thumbnail ) {
            $row->{has_thumbnail} = 1;
            my $height = $thumb_height || $default_thumb_height || 45;
            my $width  = $thumb_width  || $default_thumb_width  || 45;
            my $square = $height == 45 && $width == 45;
            @$meta{qw( thumbnail_url thumbnail_width thumbnail_height )}
                = $obj->thumbnail_url(
                Height => $height,
                Width  => $width,
                Square => $square
                );

            $meta->{thumbnail_width_offset}
                = int( ( $width - $meta->{thumbnail_width} ) / 2 );
            $meta->{thumbnail_height_offset}
                = int( ( $height - $meta->{thumbnail_height} ) / 2 );

            if ( $default_preview_width && $default_preview_height ) {
                @$meta{qw( preview_url preview_width preview_height )}
                    = $obj->thumbnail_url(
                    Height => $default_preview_height,
                    Width  => $default_preview_width,
                    );
                $meta->{preview_width_offset} = int(
                    ( $default_preview_width - $meta->{preview_width} ) / 2 );
                $meta->{preview_height_offset}
                    = int(
                    ( $default_preview_height - $meta->{preview_height} )
                    / 2 );
            }
        }
        else {
            $row->{has_thumbnail} = 0;
        }

        my $ts = $obj->created_on;
        if ( my $by = $obj->created_by ) {
            my $user = MT::Author->load($by);
            $row->{created_by}
                = $user ? $user->name : $app->translate('(user deleted)');
        }
        if ($ts) {
            $row->{created_on_formatted}
                = format_ts( MT::App::CMS::LISTING_DATE_FORMAT(),
                $ts, $blog,
                $app->user ? $app->user->preferred_language : undef );
            $row->{created_on_time_formatted}
                = format_ts( MT::App::CMS::LISTING_TIMESTAMP_FORMAT(),
                $ts, $blog,
                $app->user ? $app->user->preferred_language : undef );
            $row->{created_on_relative} = relative_date( $ts, time, $blog );
        }

        @$row{ keys %$meta } = values %$meta;
        $row->{metadata_json} = MT::Util::to_json($meta);
        $row;
    };
}

1;
